@@ -16,7 +16,8 @@ from ruamel.yaml.error import (MarkedYAMLError, MarkedYAMLFutureWarning,
 from ruamel.yaml.nodes import *                               # NOQA
 from ruamel.yaml.nodes import (SequenceNode, MappingNode, ScalarNode)
 from ruamel.yaml.compat import (utf8, builtins_module, to_str, PY2, PY3,  # NOQA
-                                ordereddict, text_type, nprint, version_tnf, Hashable)
+                                ordereddict, text_type, nprint, version_tnf, Hashable,
+                                MutableSequence)
 from ruamel.yaml.comments import *                               # NOQA
 from ruamel.yaml.comments import (CommentedMap, CommentedOrderedMap, CommentedSet,
                                   CommentedKeySeq, CommentedSeq, TaggedScalar)
@@ -247,11 +248,19 @@ class BaseConstructor(object):
         # type: (Any, Any, Any, Any, Any) -> None
         if key in mapping:
             if not self.allow_duplicate_keys:
+                mk = mapping.get(key)
+                if PY2:
+                    if isinstance(key, unicode):
+                        key = key.encode('utf-8')
+                    if isinstance(value, unicode):
+                        value = value.encode('utf-8')
+                    if isinstance(mk, unicode):
+                        mk = mk.encode('utf-8')
                 args = [
                     'while constructing a mapping',
                     node.start_mark,
                     'found duplicate key "{}" with value "{}" '
-                    '(original value: "{}")'.format(key, value, mapping.get(key)),
+                    '(original value: "{}")'.format(key, value, mk),
                     key_node.start_mark,
                     """
                     To suppress this check see:
@@ -271,6 +280,9 @@ class BaseConstructor(object):
         # type: (Any, Any, Any, Any, Any) -> None
         if key in setting:
             if not self.allow_duplicate_keys:
+                if PY2:
+                    if isinstance(key, unicode):
+                        key = key.encode('utf-8')
                 args = [
                     'while constructing a set',
                     node.start_mark,
@@ -1288,7 +1300,7 @@ class RoundTripConstructor(SafeConstructor):
         # type: () -> None
         pass
 
-    def construct_mapping(self, node, maptyp=None, deep=False):  # type: ignore
+    def construct_mapping(self, node, maptyp, deep=False):  # type: ignore
         # type: (Any, Any, bool) -> Any
         if not isinstance(node, MappingNode):
             raise ConstructorError(
@@ -1311,7 +1323,7 @@ class RoundTripConstructor(SafeConstructor):
             key = self.construct_object(key_node, deep=True)
             # lists are not hashable, but tuples are
             if not isinstance(key, Hashable):
-                if isinstance(key, list):
+                if isinstance(key, MutableSequence):
                     key_a = CommentedKeySeq(key)
                     if key_node.flow_style is True:
                         key_a.fa.set_flow_style()
